## ----------------------------------------------------------------------------
# Rishu::Remote
# -----------------------------------------------------------------------------
# 里珠・外部のプロセスの受け側.
# -----------------------------------------------------------------------------
# Mastering programed by YAMASHINA Hio
#
# Copyright 2002 YMIRLINK,Inc.
# -----------------------------------------------------------------------------
# $Id: Remote.pm,v 1.7 2002/11/10 14:00:27 hio Exp $
# -----------------------------------------------------------------------------
package Rishu::Remote;
use strict;

# -----------------------------------------------------------------------------
# Rishu::Remote->new
sub new
{
  my $pkg = shift;
  my $this = bless{},$pkg;
  
  my ($rd,$wd) = (\*STDIN,\*STDOUT);
  select((select($wd),$|=1)[0]);
  
  $this->{rd} = $rd;
  $this->{wd} = $wd;
  binmode($rd);
  binmode($wd);
  
  $this->{on_load}    = \&main::on_load;
  $this->{on_request} = \&main::on_request;
  $this->{on_unload}  = \&main::on_unload;
  $this->{on_exit} = sub{};
  
  $this->{logbits}{system} = 1;
  $this->{logbits}{recv} = 1;
  $this->{logbits}{send} = 1;
  
  $this;
}

# -----------------------------------------------------------------------------
# Rishu::Remote->run()
sub run
{
  my $this = shift;
  
  my $rd = $this->{rd};
  $this->log("system","run\n");
  for(;;)
  {
    # コマンド行の捜索.
    $_ = <$rd>;
    if( !defined($_) )
    {
      # EOF.
      last;
    }
    s/\r?\n//;
    $this->{logbits}{recv} and $this->log("recv","$_\n");
    if( $_ eq '' )
    {
      # 空行が先行してたらそれは無視.
      next;
    }
    
    # ヘッダ部分の認識
    my $requestLine = $_;
    my ($method,$type,$proto,$ver) = m,^(PROXY)\s+(\S+)\s+([^\s/]+)/(\d+\.\d+)$,i;
    
    my $in = '';
    while(<$rd>)
    {
      s/\r?\n//;
      $this->{logbits}{recv} and $this->log("recv","$_\n");
      $in .= $_."\r\n";
      if( $_ eq '' )
      {
	# ヘッダの終わりの認識.
	last;
      }
    }
    if( !$type )
    {
      $this->replyBadRequest("invalid request line: $requestLine");
      next;
    }
    $type = uc($type);
    if( $type eq 'EXIT' )
    { # そのまま終了.
      return $this->{on_exit}->();
    }elsif( $type eq 'LOAD' )
    {
      my $dir = join '',grep{s/^Directory: //} split(/\r?\n/,$in);
      if( $dir ne '' )
      {
        my $res = $this->{on_load}->($dir);
	my $reply;
	if( $res )
	{
	  $reply = "RISHU/1.1 200 OK\r\n";
	  $reply .= "Sender: Rishu/Proxy\r\n";
	}else
	{
	  $reply = "RISHU/1.1 500 Internal Server Error\r\n";
	  $reply .= "Sender: Rishu/Proxy\r\n";
	}
	if( !defined($res) || length($res)==0 )
	{
	  $reply .= "\r\n";
        }else
	{
	  $reply .= "Content-Length: ".length($res)."\r\n";
	  $reply .= "\r\n";
	  $reply .= $res;
	}
        $this->_print($reply);
      }else
      {
	$this->replyBadRequest("LOAD, no Directory: element");
      }
    }elsif( $type eq 'REQUEST' )
    {
      $in =~ s/^Command:\s+//;
      my $res = $this->{on_request}->($in);
      $this->_print("RISHU/1.1 200 OK\r\nOriginal-Response: ".join("\r\n",split(/\r?\n/,$res),"\r\n"));
    }elsif( $type eq 'UNLOAD' )
    {
      my $res = $this->{on_unload}->($in);
      my $reply;
      if( $res )
      {
	$reply = "RISHU/1.1 200 OK\r\n";
	$reply .= "Sender: Rishu/Proxy\r\n";
      }else
      {
	$reply = "RISHU/1.1 500 Internal Server Error\r\n";
	$reply .= "Sender: Rishu/Proxy\r\n";
      }
      if( !defined($res) || length($res)==0 )
      {
	$reply .= "\r\n";
      }else
      {
	$reply .= "Content-Length: ".length($res)."\r\n";
	$reply .= "\r\n";
	$reply .= $res;
      }
      $this->_print($reply);
    }else
    {
      $this->replyBadRequest("type: $type");
    }
  }
  
  $this;
}

# -----------------------------------------------------------------------------
# $remote->replyBadRequest
sub replyBadRequest
{
  my $this = shift;
  my $cause = shift;

  my $reply = "RISHU/1.1 400 Bad Request\r\n";
  $reply .= "Sender: Rishu/Proxy\r\n";
  $reply .= "Rishu-Message: $cause\r\n";
  $reply .= "\r\n";
  $this->_print($reply);
  $this;
}

# -----------------------------------------------------------------------------
# $remote->_print
sub _print
{
  my $this = shift;
  my $sym = $this->{wd};
  my $msg = join('',@_);
  print $sym $msg;
  $this->{logbits}{send} and $this->log('send',$msg);
  
  $this;
}

# -----------------------------------------------------------------------------
# $remote->log('type','text');
sub log
{
  my $this = shift;
  my $type = shift;
  my $msg = shift;
  
  if( !$this->{logbits}{$type} )
  {
    return $this;
  }
  
  # not implemented yet.
  if( !$this->{logfilename} )
  {
    my $fn = $0;
    $fn =~ s/\.pl$/_log.txt/ or $fn .= "_log.txt";  
    $this->{logfilename} = $fn;
    if( -e $fn )
    {
      (my $bak = $fn)=~s/txt$/bak/;
      rename($fn,$bak);
    }
  }
  
  if( open(LOG,">>$this->{logfilename}") )
  {
    use Time::HiRes;
    my ($timesec,$usec) = Time::HiRes::gettimeofday();
    #   0    1    2     3     4    5     6     7     8
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
	= localtime($timesec);
    ++$mon;
    $year += 1900;
    my $time = sprintf("%02d/%02d %02d:%02d:%2d.%06d",
		       $mon,$mday,$hour,$min,$sec,$usec);
    
    binmode(LOG);
    
    my $head = "$time $type> ";
    $msg =~ s/\n(?=.)/\n$head/sg;
    print LOG $head.$msg;
    close LOG;
  }
  
  $this;
}

# -----------------------------------------------------------------------------
# end of package
# -----------------------------------------------------------------------------
1;
